#ifdef VERTEX_TEXTURE_COORD
in vec2 TexCoord;
#endif

#ifdef MATERIAL_ALBEDO_MAP
uniform sampler2D uAlbedoTexture;
#endif

#ifdef MATERIAL_NORMAL_MAP
uniform sampler2D uNormalTexture;
in vec3 LightPosTS;
in vec3 PosTS;
#endif

#ifdef VERTEX_COLOR
in vec4 VertexColor;
#endif

#ifdef MATERIAL_COLOR
in vec4 MaterialColor;
#endif

out vec4 FragColor;


void main() {
    vec4 result = vec4(1.0);

    #ifdef MATERIAL_NORMAL_MAP
    vec3 normalColor = texture(uNormalTexture, TexCoord).rgb;
    vec3 normalTS = normalize(normalColor * 2.0 - 1.0);
    vec3 lightDir = normalize(LightPosTS - PosTS);
    float lightness = max(0.0, dot(lightDir, normalTS));
    result.rgb *= lightness;
    #endif

    #ifdef MATERIAL_ALBEDO_MAP
    result *= texture(uAlbedoTexture, TexCoord);
    #endif

    #ifdef VERTEX_COLOR
    result *= VertexColor;
    #endif

    #ifdef MATERIAL_COLOR
    result *= MaterialColor;
    #endif

    FragColor = result;
}