layout (location = 1) in vec2 aPos;

#ifdef VERTEX_TEXTURE_COORD
layout (location = 2) in vec2 aTexCoord;
out vec2 TexCoord;
#endif

#ifdef VERTEX_COLOR
layout (location = 3) in vec4 aVertexColor;
out vec4 VertexColor;
#endif

#ifdef MATERIAL_NORMAL_MAP
uniform vec3 uLightPos;
uniform vec3 uViewPos;

layout (location = 4) in vec2 aTangent;

out vec3 LightPosTS;
out vec3 ViewPosTS;
out vec3 PosTS;
#endif

uniform mat3 uModel;
uniform mat3 uProjectionViewModel;

#ifdef MATERIAL_COLOR
uniform vec4 uMaterialColor;
out vec4 MaterialColor;
#endif

void main()
{
    #ifdef MATERIAL_NORMAL_MAP
    mat3 normalMatrix = transpose(inverse(mat3(uModel)));
    vec3 T = normalize(normalMatrix*vec3(aTangent, 0.0));
    vec3 N = normalize(normalMatrix*vec3(0.0, 0.0, 1.0));
    T = normalize(T - dot(T, N) * N);
    vec3 B = cross(N, T);

    mat3 TBN = transpose(mat3(T, B, N));
    vec3 pos = uModel * vec3(aPos, 1.0);
    LightPosTS = TBN * uLightPos;
    ViewPosTS = TBN * uViewPos;
    PosTS = TBN * pos;
    #endif

    gl_Position = vec4(uProjectionViewModel * vec3(aPos, 1.0), 1.0);

    #ifdef VERTEX_TEXTURE_COORD
    TexCoord = aTexCoord;
    #endif

    #ifdef VERTEX_COLOR
    VertexColor = aVertexColor;
    #endif

    #ifdef MATERIAL_COLOR
    MaterialColor = uMaterialColor;
    #endif
}